@@ -6,6 +6,7 @@
 #include <sys/stat.h>
 
 #include "gtkscrollpane.h"
+#include "gtkchecklist.h"
 #include "gtkgs.h"
 #include "ggvutils.h"
 #include "prefs.h"
@@ -21,7 +22,7 @@ void recent_callback(GtkWidget *w, gpointer *data)
 
         if (stat((char*)wwd->data, &stat_rec) == 0) {
                 if (S_ISREG(stat_rec.st_mode)) {
-                        wwd->ggv->loaded = load_gs(wwd->ggv, (char*)wwd->data);
+                        load_gs(wwd->ggv, (char*)wwd->data);
                         return;
                 }
                 error_msg = g_strdup_printf(_("%s is not a regular file."), (char*)wwd->data);
@@ -34,44 +35,81 @@ void recent_callback(GtkWidget *w, gpointer *data)
         error_message(wwd->ggv,error_msg);
 }
 
+
 void unmark_all_pages_callback(GtkWidget *widget, gpointer data) 
 {
+        gint i, n;
         ggv_window *ggv = (ggv_window *) data;
-        g_return_if_fail (ggv!=NULL);
 
-        toggle_pages(ggv, UNMARK_ALL_PAGES, GTK_GS(ggv->gs)->current_page);
+        g_return_if_fail (ggv!=NULL);
+        if((!GTK_GS(ggv->gs)->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
+                return;
+
+        n = gtk_gs_document_numpages (GTK_GS (ggv->gs));
+        gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
+        for (i = 0; i < n; i++)
+                gtk_check_list_set_toggled (GTK_CHECK_LIST (ggv->pagelist), i, FALSE);
+        gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
 }
 
 void toggle_current_page_callback(GtkWidget *widget, gpointer data) 
 {
         ggv_window *ggv = (ggv_window *) data;
-        g_return_if_fail (ggv!=NULL);
 
-        toggle_pages(ggv, TOGGLE_CURRENT_PAGE, GTK_GS(ggv->gs)->current_page);
+        g_return_if_fail (ggv!=NULL);
+        if((!GTK_GS(ggv->gs)->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
+                return;
+
+        gtk_check_list_toggle_row (GTK_CHECK_LIST (ggv->pagelist),
+                                   GTK_GS (ggv->gs)->current_page);
 }
 
 void toggle_even_pages_callback(GtkWidget *widget, gpointer data) 
 {
+        gint i, n;
         ggv_window *ggv = (ggv_window *) data;
-        g_return_if_fail (ggv!=NULL);
 
-        toggle_pages(ggv, TOGGLE_EVEN_PAGES, 0);
+        g_return_if_fail (ggv!=NULL);
+        if((!GTK_GS(ggv->gs)->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
+                return;
+
+        n = gtk_gs_document_numpages (GTK_GS (ggv->gs));
+        gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
+        for (i = 0; i < n; i++)
+                gtk_check_list_set_toggled (GTK_CHECK_LIST (ggv->pagelist), i, i % 2);
+        gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
 }
 
 void toggle_odd_pages_callback(GtkWidget *widget, gpointer data) 
 {
+        gint i, n;
         ggv_window *ggv = (ggv_window *) data;
-        g_return_if_fail (ggv!=NULL);
 
-        toggle_pages(ggv, TOGGLE_ODD_PAGES, 0);
+        g_return_if_fail (ggv!=NULL);
+        if((!GTK_GS(ggv->gs)->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
+                return;
+
+        n = gtk_gs_document_numpages (GTK_GS (ggv->gs));
+        gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
+        for (i = 0; i < n; i++)
+                gtk_check_list_set_toggled (GTK_CHECK_LIST (ggv->pagelist), i, !(i % 2));
+        gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
 }
 
 void toggle_all_pages_callback(GtkWidget *widget, gpointer data) 
 {
+        gint i, n;
         ggv_window *ggv = (ggv_window *) data;
-        g_return_if_fail (ggv!=NULL);
 
-        toggle_pages(ggv, TOGGLE_ALL_PAGES, 0);
+        g_return_if_fail (ggv!=NULL);
+        if((!GTK_GS(ggv->gs)->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
+                return;
+
+        n = gtk_gs_document_numpages (GTK_GS (ggv->gs));
+        gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
+        for (i = 0; i < n; i++)
+                gtk_check_list_set_toggled (GTK_CHECK_LIST (ggv->pagelist), i, TRUE);
+        gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
 }
 
 static void about_destroy_callback(GtkObject *obj, GtkWidget **about)
@@ -96,17 +134,19 @@ void about_callback(GtkWidget *widget, gpointer data)
                 "Daniel M. German <dmg@csg.uwaterloo.ca> (current maintainer),",
                 "Werner Koerner <werner.koerner@zae.uni-wuerzburg.de>",
                 "Tuomas J. Lukka <lukka@iki.fi>",
-                "Jaka Mocnik <jaka.mocnik@kiss.uni-lj.si>",
+                "Jaka Mocnik <jaka@gnu.org>",
                 "and the whole GNOME developers team.",
                 "",
                 "GGV would be a carcass without Ghostscript, written by Peter Deutsch.",
 #else
                 "Jonathan Blandford <jrb@redhat.com>",
-                "Daniel M. German <dmg@csg.uwaterloo.ca> (current maintainer),",
+                "Daniel M. German <dmg@csg.uwaterloo.ca>,",
                 "Werner Koerner <werner.koerner@zae.uni-wuerzburg.de>",
                 "Tuomas J. Lukka <lukka@iki.fi>",
-                "Jaka Mocnik <jaka.mocnik@kiss.uni-lj.si>",
+                "Jaka Mocnik <jaka@gnu.org>  (current maintainer)",
+                "Johannes Plass <plass@thep.physik.uni-mainz.de>",
 		"Istvan Szekeres <szekeres@cyberspace.mht.bme.hu>",
+                "Tim Theisen <tim@cs.wisc.edu>",
 #endif
 		NULL
 	};
@@ -117,8 +157,8 @@ void about_callback(GtkWidget *widget, gpointer data)
         gtk_widget_push_visual (gdk_imlib_get_visual ());
         gtk_widget_push_colormap (gdk_imlib_get_colormap ());
         
-	about = gnome_about_new(_("GNOME Ghostview"), VERSION,
-				"Copyright (C) 1998-2000 the Free Software Foundation",
+	about = gnome_about_new(_("Gnome Ghostview"), VERSION,
+				"Copyright (C) 1998-2001 the Free Software Foundation",
 				(const gchar **) authors,
 				_("PostScript(TM) document viewer.\n"
                                   "Based on Tim Theisen's excellent Ghostview application."),
@@ -133,7 +173,7 @@ void about_callback(GtkWidget *widget, gpointer data)
 
 void new_callback(GtkWidget *widget, gpointer data)
 {
-        open_window(NULL);
+        open_window(NULL, -1, -1, -1, -1);
 }
 
 void button_press_callback(GtkWidget *widget, GdkEventButton *event,
@@ -179,15 +219,36 @@ void button_press_callback(GtkWidget *widget, GdkEventButton *event,
 #endif
               
                 break;
+                /* Support for wheel mice */
         case 3:
                 gnome_popup_menu_do_popup(ggv->popup_menu, NULL, NULL, event, ggv);
                 if(ggv->menus_vis_pu)
                         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->menus_vis_pu),
                                                        ggv->show_menus);
+                if(ggv->toolbar_vis_pu)
+                        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->toolbar_vis_pu),
+                                                       ggv->show_toolbar);
                 if(ggv->panel_vis_pu)
                         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->panel_vis_pu),
                                                        ggv->show_panel);
                 break;
+                /* Support for wheel mice */
+	case 4:
+		if(event->state & GDK_CONTROL_MASK)
+			gtk_gs_scroll(GTK_GS(ggv->gs), -64, 0);
+		else if(event->state & GDK_SHIFT_MASK)
+			zoom_to(ggv, ggv->zoom_magstep+1);
+		else
+			gtk_gs_scroll(GTK_GS(ggv->gs), 0, -64);
+		break;
+	case 5:
+		if(event->state & GDK_CONTROL_MASK)
+			gtk_gs_scroll(GTK_GS(ggv->gs), 64, 0);
+		else if(event->state & GDK_SHIFT_MASK)
+			zoom_to(ggv, ggv->zoom_magstep-1);
+		else
+			gtk_gs_scroll(GTK_GS(ggv->gs), 0, 64);
+		break;
         default:
                 break;
         }
@@ -228,20 +289,29 @@ gint exit_callback(GtkWidget *widget, gpointer data)
         GList *node;
 
         /* do we want to save anything before we exit??? */        
+
+
         while (window_list) {
+
+
                 node = window_list;
                 window_list = window_list->next;
                 if(window_list)
                         window_list->prev = NULL;
                 node->next = NULL;
                 ggv = (ggv_window *)node->data;
+
+                /* save size of the current window */
+                gdk_window_get_size (GTK_WIDGET(ggv->main_window)->window,
+                                     &ggv_default_width,
+                                     &ggv_default_height);
+
                 g_list_free(node);
                 gtk_widget_destroy (GTK_WIDGET (ggv->main_window));
                 g_free(ggv);
         }
 
-	gtk_main_quit();
-
+	gtk_main_quit ();
 	return 0;
 }
 
@@ -312,12 +382,103 @@ void file_open_ok_callback(GtkWidget *widget, gpointer data)
         gtk_widget_destroy(ggv->file_sel);
         ggv->file_sel = NULL;
 
-        if ((ggv->loaded = load_gs(ggv, file)))
+        if ((GTK_GS(ggv->gs)->loaded = load_gs(ggv, file)))
                 recent_update();
 
         g_free(file);
 }
 
+void
+watch_file_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+        GtkGS *gs;
+        gs = GTK_GS(ggv->gs);
+        
+        gs->watch_doc = GTK_CHECK_MENU_ITEM(widget)->active;
+
+	if(!gs->watch_doc) {
+		if(gs->timer_tag)
+                        gtk_timeout_remove(gs->timer_tag);
+	}
+        else {
+		gs->timer_tag = gtk_timeout_add(GGV_WATCH_INTERVAL,
+						 timer_callback, ggv);
+        }
+}
+
+void
+antialiasing_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+        
+        GtkGS *gs;
+        gs = GTK_GS(ggv->gs);
+        
+        gs->antialiased = GTK_CHECK_MENU_ITEM(widget)->active;
+
+        /* We assume that if it gets here is because it was toggled,
+           so no need to check what is the previous state of gs->antialiased */
+
+        reload_callback(NULL, ggv);
+
+}
+
+void
+override_paper_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+        
+        GtkGS *gs;
+        gs = GTK_GS(ggv->gs);
+        
+        gs->override_media = GTK_CHECK_MENU_ITEM(widget)->active;
+
+        gtk_gs_set_pagemedia(gs, -1, gs->current_page);
+
+        if (GTK_GS(ggv->gs)->loaded)
+                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page);
+}
+
+void
+override_orientation_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+        
+        GtkGS *gs;
+        gs = GTK_GS(ggv->gs);
+        
+        g_print("Overriding orientation GS [%d]\n", GTK_CHECK_MENU_ITEM(widget)->active);
+
+        gtk_gs_set_override_orientation (gs, GTK_CHECK_MENU_ITEM(widget)->active);
+
+        /* We assume that if it gets here is because it was toggled,
+           so no need to check what is the previous state of gs->antialiased */
+
+        if (GTK_GS(ggv->gs)->loaded)
+                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page);
+}
+
+void orientation_callback(GtkWidget *widget, gpointer data)
+{
+        window_with_data *wwd = (window_with_data *) data;
+        ggv_window *ggv = wwd->ggv;
+
+        if (!GTK_CHECK_MENU_ITEM(widget)->active){
+                /* This is a deactivate signal Do nothing*/
+                return;
+        }
+
+        /* Set the new orientation */
+        g_print("Setting fallback orientation GS [%d]\n",GPOINTER_TO_INT(wwd->data));
+      
+        gtk_gs_set_orientation (GTK_GS(ggv->gs), GPOINTER_TO_INT(wwd->data));
+
+        if (GTK_GS(ggv->gs)->loaded)
+                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page);
+}
+
+
 void show_menubar_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
@@ -343,6 +504,31 @@ void show_menubar_callback(GtkWidget *widget, gpointer data)
                                                ggv->show_menus);
 }
 
+void show_toolbar_callback(GtkWidget *widget, gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+        
+	GnomeDockItem* dock_item = gnome_app_get_dock_item_by_name(GNOME_APP(ggv->main_window), 
+				GNOME_APP_TOOLBAR_NAME);
+
+	ggv->show_toolbar = GTK_CHECK_MENU_ITEM(widget)->active;
+	
+	
+	g_assert(dock_item != NULL);
+
+	if (ggv->show_toolbar) {
+                gtk_widget_show (GTK_WIDGET(dock_item));
+        } else {
+                gtk_widget_hide (GTK_WIDGET(dock_item));
+		gtk_widget_queue_resize (ggv->main_window);
+        }
+        if(ggv->toolbar_vis &&
+           GTK_CHECK_MENU_ITEM(ggv->toolbar_vis)->active != ggv->show_toolbar)
+                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->toolbar_vis),
+                                               ggv->show_toolbar);
+}
+
+
 void hide_panel_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
@@ -396,7 +582,7 @@ void key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event,
         gint key = event->keyval;
         gint orientation = gtk_gs_get_orientation (GTK_GS (ggv->gs));
 
-        if (!ggv->loaded)
+        if (!GTK_GS(ggv->gs)->loaded)
                 return;
         /*ugh. the possibilities! */
         switch (key) {
@@ -498,7 +684,7 @@ void key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event,
 void next_page_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
-        if (ggv->loaded)
+        if (GTK_GS(ggv->gs)->loaded)
 	        goto_page(ggv, GTK_GS(ggv->gs)->current_page + 1);
 }
 
@@ -506,7 +692,7 @@ void previous_page_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        if (ggv->loaded)
+        if (GTK_GS(ggv->gs)->loaded)
 		goto_page(ggv, GTK_GS(ggv->gs)->current_page - 1);
 }
 
@@ -541,17 +727,6 @@ void open_callback(GtkWidget *widget, gpointer data)
         gtk_widget_show(ggv->file_sel);
 }
 
-void orientation_callback(GtkWidget *widget, gpointer data)
-{
-        window_with_data *wwd = (window_with_data *) data;
-        ggv_window *ggv = wwd->ggv;
-
-        if (!ggv->loaded)
-                return;
-        if (gtk_gs_set_orientation (GTK_GS(ggv->gs), GPOINTER_TO_INT(wwd->data)))
-                gtk_gs_goto_page (GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page);
-}
-
 void preferences_callback(GtkWidget *widget, gpointer data) {
         open_prefs_dialog(&prefs_dialog);
 }
@@ -562,13 +737,13 @@ void print_callback(GtkWidget *widget, gpointer data)
         gchar *print_command;
         GtkGS *gs = GTK_GS(ggv->gs);
 
-        if(ggv->loaded) {
+        if(GTK_GS(ggv->gs)->loaded) {
                 /* make sure we print the _uncompressed_ file in case it
 		 * the loaded one was compressed!
                  * Quote the filename to allow special characters.
 		 */
  		gchar *filename;
-                filename = ggv_quote_filename (PS_FILE (gs));
+                filename = ggv_quote_filename (GTK_GS_GET_PS_FILE(gs));
  		print_command = g_strdup_printf(gs_print_cmd, filename);
                 g_free (filename);
 
@@ -584,7 +759,33 @@ void print_marked_pages_callback(GtkWidget *widget, gpointer data)
         ggv_window *ggv = (ggv_window *)data;
         gchar *tmp_name, *print_command;
 
-        if(ggv->loaded) {
+        if(GTK_GS(ggv->gs)->loaded) {
+                if(gtk_gs_count_marked_pages(GTK_GS(ggv->gs)) == 0) {
+			GtkWidget *dialog, *label;
+         		int button;
+
+        		dialog = gnome_message_box_new (
+                 		_("There are no marked pages.\nDo you want to print the entire document?"),
+				GNOME_MESSAGE_BOX_QUESTION,
+                 		GNOME_STOCK_BUTTON_YES,
+                 		GNOME_STOCK_BUTTON_NO,
+                 		NULL);
+			
+		 	if (ggv)
+                 		gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW(ggv->main_window));
+		
+			/*gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);*/
+
+			gnome_dialog_set_default(GNOME_DIALOG(dialog), GNOME_NO);
+
+         		button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+			
+			if(button == GNOME_YES)
+				print_callback(widget, data);
+
+	                return;
+	        }
+
                 tmp_name = g_malloc(256);
                 tmpnam(tmp_name);
 
@@ -606,7 +807,7 @@ void recenter_page_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        if (ggv->loaded)
+        if (GTK_GS(ggv->gs)->loaded)
                 gtk_gs_center_page (GTK_GS (ggv->gs));
 }
 
@@ -614,9 +815,9 @@ void reload_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        if (ggv->loaded) {
+        if (GTK_GS(ggv->gs)->loaded) {
                 gchar *fname = g_strdup (GTK_GS (ggv->gs)->gs_filename);
-                ggv->loaded = load_gs (ggv, fname);
+                GTK_GS(ggv->gs)->loaded = load_gs (ggv, fname);
                 g_free (fname);
         }
 }
@@ -685,7 +886,7 @@ void save_callback(GtkWidget *widget, gpointer data) {
         char *error_msg;
         gchar *dirname;
 
-        if (ggv->loaded) {
+        if (GTK_GS(ggv->gs)->loaded) {
                 if(ggv->save_file_sel)
                         return;
 
@@ -723,7 +924,7 @@ void save_callback(GtkWidget *widget, gpointer data) {
 void save_as_callback(GtkWidget *widget, gpointer data) 
 {
         ggv_window *ggv = (ggv_window *) data;
-        if (ggv->loaded) {
+        if (GTK_GS(ggv->gs)->loaded) {
         }
 }
 
@@ -732,7 +933,7 @@ void scrollpane_middle_click_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        if (ggv->loaded) {
+        if (GTK_GS(ggv->gs)->loaded) {
 		goto_page( ggv, GTK_GS(ggv->gs)->current_page - 1);
         }
 }
@@ -741,29 +942,19 @@ void scrollpane_right_click_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        if (ggv->loaded) {
+        if (GTK_GS(ggv->gs)->loaded) {
 		goto_page( ggv, GTK_GS(ggv->gs)->current_page + 1);
         }
 }
 
-void select_page_button_press_callback(GtkWidget *widget,
-				       GdkEventButton *event,
-				       gpointer data)
+void
+page_toggled_callback (GtkWidget *widget, gint row, gboolean toggled, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
-        gint row = 0;
-        gint col = 0;
-
-        if (GTK_CLIST(ggv->pagelist)->clist_window == event->window) {
-                gtk_clist_get_selection_info (GTK_CLIST(widget), event->x, event->y, &row, &col);
-                if ((event->button == 3) && (row >= 0)) {
-                        toggle_pages(ggv, TOGGLE_CURRENT_PAGE, row);
-                }
-        }
 
+        GTK_GS(ggv->gs)->pages_marked[row] = toggled;
 }
 
-
 void select_page_callback(GtkWidget *widget, gint row, gint column,
 				 GdkEventButton *event, gpointer data)
 {
@@ -783,8 +974,8 @@ void select_page_callback(GtkWidget *widget, gint row, gint column,
 }
 
 void drop_callback(GtkWidget *widget, GdkDragContext *context,
-                          gint x, gint y, GtkSelectionData *selection_data,
-                          guint info, guint time, gpointer data) {
+                   gint x, gint y, GtkSelectionData *selection_data,
+                   guint info, guint time, gpointer data) {
 	GList *names, *list;
         ggv_window *ggv = (ggv_window *)data;
 
@@ -792,10 +983,11 @@ void drop_callback(GtkWidget *widget, GdkDragContext *context,
 	case TARGET_URI_LIST:
 		list = names = gnome_uri_list_extract_filenames (selection_data->data);
 		while (names) {
-                        if(ggv->loaded)
-                                open_window(names->data);
+                        if(GTK_GS(ggv->gs)->loaded) {
+                                open_window(names->data, -1, -1, -1, -1);
+                        }
                         else if (load_gs (ggv, names->data))
-                                ggv->loaded = TRUE;
+                                GTK_GS(ggv->gs)->loaded = TRUE;
 
 			names = names->next;
 		}
@@ -806,26 +998,69 @@ void drop_callback(GtkWidget *widget, GdkDragContext *context,
 	}
 }
 
+void paper_callback(GtkWidget *widget, gpointer *data)
+{
+        window_with_data *wwd = (window_with_data *) data;
+        ggv_window *ggv = wwd->ggv;
+        GtkGS *gs = GTK_GS(ggv->gs);
+
+        if (!GTK_CHECK_MENU_ITEM(widget)->active){
+                /* This is a deactivate signal */
+                return;
+        }
+
+        gs->default_page_media = GPOINTER_TO_INT(wwd->data);
+        gtk_gs_set_pagemedia(gs, -1, gs->current_page);
+}
+
 void zoom_callback(GtkWidget *widget, gpointer data)
 {
         window_with_data *wwd = (window_with_data *) data;
         ggv_window *ggv = wwd->ggv;
+        int new_mag = GPOINTER_TO_INT(wwd->data);
 
-        if (ggv->zoom_magstep == GPOINTER_TO_INT(wwd->data)) 
+
+        if (!GTK_CHECK_MENU_ITEM(widget)->active){
+                /* This is a deactivate signal */
                 return;
-	zoom_to(ggv, GPOINTER_TO_INT(wwd->data));
+        }
+
+        if (ggv->zoom_magstep == new_mag) {
+                return;
+        }
+        zoom_to(ggv, new_mag);
 }
 
 void zoomin_callback(GtkWidget *widget, gpointer *data)
 {
         ggv_window *ggv = (ggv_window *)data;
-	zoom_to(ggv, ggv->zoom_magstep+1);
+
+        /* if there is an option in the zoom menu, we set it,
+           that will zoom to the corresponding zoom mag. step
+           as a side effect. */
+
+        /* If it does not exist (the function returned FALSE)
+           then do the call explicitly */
+ 
+        if (!set_corresponding_zoom_menu_option(ggv, ggv->zoom_magstep+1))
+                zoom_to(ggv, ggv->zoom_magstep+1);
 }
 
+
 void zoomout_callback(GtkWidget *widget, gpointer *data)
 {
         ggv_window *ggv = (ggv_window *)data;
-	zoom_to(ggv, ggv->zoom_magstep-1);
+
+        /* if there is an option in the zoom menu, we set it,
+           that will zoom to the corresponding zoom mag. step
+           as a side effect. */
+
+        /* If it does not exist (the function returned FALSE)
+           then do the call explicitly */
+
+
+        if (!set_corresponding_zoom_menu_option(ggv, ggv->zoom_magstep-1))
+                zoom_to(ggv, ggv->zoom_magstep-1);
 }
 
 void prefs_destroy_callback(GtkWidget *w, ggv_prefs *p)
@@ -859,14 +1094,36 @@ void prefs_apply_callback(GtkWidget *w, gint page, ggv_prefs *p)
         active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(p->media))));
         for(i = 0; i < PAPER_SIZE_COUNT; i++)
                 if(active == p->media_choice[i]) {
-                        gs_def_media = i;
+                        gtk_gs_set_default_page_media(i);
                         break;
                 }
-        
-        gs_watch_doc = GTK_TOGGLE_BUTTON(p->watch)->active;
-        gs_antialiased = GTK_TOGGLE_BUTTON(p->aa)->active;
-        gs_override_media = GTK_TOGGLE_BUTTON(p->override_media)->active;
+
+        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(p->zoom))));
+        for(i = 0; i < MENU_ZOOM_SIZE - 1; i++){
+                if(active == p->zoom_choice[i]) {
+                        gs_default_magnification = ZoomMenuMagnificationSteps[i];
+                        gtk_gs_set_default_zoom_factor(compute_zoom(gs_default_magnification));
+                        break;
+                }
+        }
+
+        active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(p->orientation))));
+        for(i = 0; i < MENU_ORIENTATION_SIZE; i++){
+                if(active == p->orientation_choice[i]) {
+                        gtk_gs_set_default_orientation(i);
+                        break;
+                }
+        }
+
+        gtk_gs_set_default_respect_eof(GTK_TOGGLE_BUTTON(p->respect_eof)->active);
+        gtk_gs_set_default_watch_doc(GTK_TOGGLE_BUTTON(p->watch)->active);
+        gtk_gs_set_default_antialiased(GTK_TOGGLE_BUTTON(p->aa)->active);
+        gtk_gs_set_default_override_media(GTK_TOGGLE_BUTTON(p->override_media)->active);
+        gtk_gs_set_default_override_orientation(GTK_TOGGLE_BUTTON(p->override_orientation)->active);
+
         gs_panel = GTK_TOGGLE_BUTTON(p->tbar)->active;
+        gs_save_geometry = GTK_TOGGLE_BUTTON(p->savegeo)->active;
+	gs_toolbar = GTK_TOGGLE_BUTTON(p->toolbar)->active;
         gs_menubar = GTK_TOGGLE_BUTTON(p->mbar)->active;
         gs_auto_jump = GTK_TOGGLE_BUTTON(p->auto_jump)->active;
 
@@ -907,7 +1164,7 @@ gint timer_callback (gpointer data)
         gs = GTK_GS (ggv->gs);
 
         /* remove timer if no valid file is loaded */
-	if (!ggv->loaded || !ggv->gs || !gs->watch_doc || !gs->gs_psfile) {
+	if (!GTK_GS(ggv->gs)->loaded || !ggv->gs || !gs->watch_doc || !gs->gs_psfile) {
                 gs->timer_tag = 0;
 		return FALSE;
 	}
