@@ -133,6 +133,19 @@ The DONE message indicates that ghostscript has finished processing.
 #define O_NONBLOCK O_NDELAY
 #endif
 
+/* Default values to pass to gtk_gs_init */
+typedef struct _GtkGSDefaults 
+{
+        gboolean   antialiased;
+        gboolean   override_media;
+        gint       default_page_media;
+	gboolean   watch_doc;                /* Automatic reload if file changed */
+        gboolean   override_orientation;
+        gboolean   respect_eof;
+        gint       fallback_orientation;
+        gfloat     zoom_factor;
+} GtkGSDefaults;
+
 typedef void (*InterpreterMessageSignal)(GtkObject *object, gchar *msg,
                                          gpointer data);
 
@@ -140,6 +153,22 @@ enum { INTERPRETER_MESSAGE, LAST_SIGNAL };
 
 static gboolean broken_pipe = FALSE;
 
+/* 
+   This variable will hold the defaults to use when creating new
+   gtk_gs widgets.
+*/
+static GtkGSDefaults gtkgs_defaults =
+{
+     TRUE,   /* Antialised */
+     TRUE,   /* override media */
+     1,      /* default page media */
+     TRUE,   /* Watch doc */
+     FALSE,  /* Override orientation */
+     FALSE,  /* Respect EOF */
+     GTK_GS_ORIENTATION_PORTRAIT,   /* Orientation */
+     1.0,   /* Zoom factor */
+};
+
 static void catchPipe(int i)
 {
         g_print ("in catch Pipe!!!\n");
@@ -156,6 +185,7 @@ static void gtk_gs_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static gint gtk_gs_widget_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 static void gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data);
 static void gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data);
+static void gtk_gs_emit_error_msg (GtkGS *gs, const gchar *msg);
 static void send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close);
 static void set_up_page (GtkGS *gs);
 static void close_pipe (int p[2]);
@@ -193,7 +223,8 @@ gtk_gs_init (GtkGS *gs)
         gs->bpixmap     = NULL;
         gs->use_bpixmap = TRUE;
 
-        gs->disable_start   = FALSE;
+        gs->current_page = -1;
+        gs->disable_start = FALSE;
         gs->interpreter_pid = -1;
 
         gs->width  = -1;
@@ -212,6 +243,7 @@ gtk_gs_init (GtkGS *gs)
         gs->send_filename_to_gs = FALSE;
 
         gs->doc = NULL;
+        gs->loaded = FALSE;
 
 	gs->interpreter_input = -1;
 	gs->interpreter_output = -1;
@@ -237,12 +269,16 @@ gtk_gs_init (GtkGS *gs)
         gs->top_margin = 0;
         gs->right_margin = 0;
         gs->bottom_margin = 0;
-        gs->orientation = GTK_GS_ORIENTATION_DEFAULT;
-        gs->zoom_factor = 1.0;
-        gs->default_page_media = 1;
-        gs->antialiased = TRUE;
-        gs->override_media = FALSE;
         gs->pages_marked = NULL;
+
+        /* Set user defined defaults */
+        gs->override_orientation = gtk_gs_get_default_override_orientation();
+        gs->fallback_orientation = gtk_gs_get_default_orientation();
+        gs->zoom_factor = gtk_gs_get_default_zoom_factor();
+        gs->default_page_media = gtk_gs_get_default_page_media();
+        gs->antialiased = gtk_gs_get_default_antialiased();
+        gs->override_media = gtk_gs_get_default_override_media();
+        gs->respect_eof = gtk_gs_get_default_respect_eof();
 }
 
 static void
@@ -354,7 +390,7 @@ gtk_gs_destroy (GtkObject *object)
  * something changes.
  */
 static void
-gtk_gs_munge_adjustments (GtkGS 	*gs)
+gtk_gs_munge_adjustments (GtkGS *gs)
 {
         gint x, y;
 
@@ -431,7 +467,6 @@ gtk_gs_realize (GtkWidget *widget)
         widget->style = gtk_style_attach (widget->style, widget->window);
         gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 
-
         /* now we set up the child window.  This is the one that ps actually draws too. */
         attributes.x = 0;
         attributes.y = 0;
@@ -440,19 +475,17 @@ gtk_gs_realize (GtkWidget *widget)
         gdk_window_set_user_data (gs->pstarget, widget);
         /* gdk_window_show (gs->pstarget); */
         gdk_window_clear (gs->pstarget);
-        gtk_style_set_background (widget->style, 
-                                  gs->pstarget,
-                                  GTK_STATE_ACTIVE);
+        gtk_style_set_background (widget->style, gs->pstarget, GTK_STATE_ACTIVE);
+
+        gs->width = 0;
+        gs->height = 0;
+
+        gtk_gs_set_pagemedia(gs, -1, 0);
 
-        /*
-          gs->width = attributes.width;
-          gs->height = attributes.height;
-        */
-	/* XXX ??? */
         if ((gs->width > 0) && (gs->height > 0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
-        /* XXX: how to handle X BadAlloc and stuff? */
+
         gtk_signal_connect(GTK_OBJECT(widget), "event",
                            GTK_SIGNAL_FUNC(gtk_gs_widget_event), gs);
 }
@@ -543,9 +576,14 @@ gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data)
         gs = GTK_GS (data);
         if (gs->bpixmap == NULL)
                 return;
-        
-        if (adjustment->value + adjustment->page_size/2 > adjustment->upper)
-                g_print ("value = %f\npage_size = %f\nupper = %f\n", adjustment->value,adjustment->page_size,adjustment->upper);
+
+#if 0
+        g_print("Adjustment %c: val = %f, page = %f, upper = %f, lower = %f\n",
+                (adjustment == gs->hadj)?'H':'V',
+                adjustment->value, adjustment->page_size,
+                adjustment->upper, adjustment->lower);
+#endif
+   
         gdk_window_get_geometry (gs->pstarget, &x, &y, &width, &height, &depth);
         if (gs->width <= gs->widget.allocation.width)
                 newx = (gs->widget.allocation.width-gs->width)/2;
@@ -663,19 +701,9 @@ set_up_page (GtkGS *gs)
         
         stop_interpreter(gs);
         
-        if (gs->orientation == GTK_GS_ORIENTATION_DEFAULT)
-                /* FIXME: hmmmm. many things don't seem to set their
-                   default orientation.... */
-                /*orientation = gs->doc->default_page_orientation;*/
-                orientation = GTK_GS_ORIENTATION_PORTRAIT;
-        else
-                orientation = gs->orientation;
-
-
+        orientation = gtk_gs_get_orientation(gs);
+        
         if ((gs->use_bpixmap) && compute_size (gs)) {
-                /*
-                g_print("Creating pixmap\n");
-                */
                 gdk_flush(); 
 
                 if (gs->bpixmap) {
@@ -696,6 +724,8 @@ set_up_page (GtkGS *gs)
                 gdk_window_set_back_pixmap(gs->pstarget, gs->bpixmap, FALSE);
 
                 gdk_window_resize (gs->pstarget, gs->width, gs->height); 
+
+                gdk_flush();
         }
 
 #ifdef HAVE_SETLOCALE
@@ -708,15 +738,18 @@ set_up_page (GtkGS *gs)
         pprivate = (GdkPixmapPrivate *) gs->bpixmap;
         snprintf(buf, 1024, "%ld %d %d %d %d %d %f %f %d %d %d %d",
                  (pprivate ? pprivate->xwindow : 0L),
-                 orientation,
-                 gs->llx,gs->lly,
-                 gs->urx,gs->ury,
+                 orientation * 90,
+                 gs->llx, gs->lly,
+                 gs->urx, gs->ury,
                  gs->xdpi * gs->zoom_factor,
                  gs->ydpi * gs->zoom_factor,
                  gs->left_margin,
                  gs->bottom_margin,
                  gs->right_margin,
                  gs->top_margin);
+
+        /*        g_print("Sent to GS: %s\n", buf); */
+
 #ifdef HAVE_SETLOCALE
         setlocale (LC_NUMERIC, savelocale);
 #endif
@@ -859,7 +892,7 @@ input (gpointer data, gint source, GdkInputCondition condition)
                                               gs->buffer_bytes_left);
 
                         if (broken_pipe) {
-                                g_print ("the pipe is broken\n");
+                                gtk_gs_emit_error_msg(gs, g_strdup(_("Broken pipe.")));
                                 broken_pipe = FALSE;
                                 interpreter_failed(gs);
                         } else if (bytes_written == -1) {
@@ -900,8 +933,6 @@ start_interpreter (GtkGS *gs)
 
 	stop_interpreter(gs);
         
-        /* FIXME: we need to clear the window first... */
-
 	if (gs->disable_start == TRUE)
                 return 0;
         
@@ -918,7 +949,7 @@ start_interpreter (GtkGS *gs)
 
         /* set up the pipes */
         if (gs->send_filename_to_gs) {
-                argv[argc++] = PS_FILE (gs);
+                argv[argc++] = GTK_GS_GET_PS_FILE(gs);
                 argv[argc++] = "-c";
                 argv[argc++] = "quit";
         }
@@ -1057,25 +1088,6 @@ stop_interpreter (GtkGS *gs)
 	gs->busy = FALSE;
 }
 
-#if 0
-static void
-Message(GtkGS *gs, Xevent event)
-{
-        gvw->ghostview.mwin = event->xclient.data.l[0];
-    if (event->xclient.message_type ==
-	XmuInternAtom(XtDisplay(w), gvc->ghostview_class.page)) {
-	gvw->ghostview.busy = False;
-	XDefineCursor(XtDisplay(gvw), XtWindow(gvw), gvw->ghostview.cursor);
-	XtCallCallbackList(w, gvw->ghostview.message_callback, "Page");
-    } else if (event->xclient.message_type ==
-	       XmuInternAtom(XtDisplay(w), gvc->ghostview_class.done)) {
-	StopInterpreter(w);
-	XtCallCallbackList(w, gvw->ghostview.message_callback, "Done");
-    }
-}
-
-#endif
-
 /* publicly accessible functions */
 
 guint
@@ -1136,9 +1148,9 @@ gtk_gs_new_from_file(GtkAdjustment *hadj, GtkAdjustment *vadj, char *fname)
 
 
 /*
- * Show error message mas -> send signal "interpreter_message"
+ * Show error message -> send signal "interpreter_message"
  */
-static void gtkgs_errmsg (GtkGS *gs, const gchar *msg)
+static void gtk_gs_emit_error_msg (GtkGS *gs, const gchar *msg)
 {
 	gtk_signal_emit (GTK_OBJECT (gs),
 			 gtk_gs_signals [INTERPRETER_MESSAGE],
@@ -1181,10 +1193,8 @@ static gchar *check_filecompressed (GtkGS *gs)
 	filename = ggv_quote_filename (gs->gs_filename);
 	filename_unc = tempnam (g_get_tmp_dir(), "unc");
 	filename_err = tempnam (g_get_tmp_dir(), "err");
-	cmdline = g_strdup_printf ("%s <%s >%s 2>%s", cmd,
+	cmdline = g_strdup_printf ("%s %s >%s 2>%s", cmd,
 				   filename, filename_unc, filename_err);
-	g_free (filename);
-
 	if ((system (cmdline) == 0)
 	    && ggv_file_readable (filename_unc)
 	    && (ggv_file_length (filename_err) == 0)) {
@@ -1194,13 +1204,13 @@ static gchar *check_filecompressed (GtkGS *gs)
 		/* report error */
 		g_snprintf (buf, 1024, _("Error while decompressing file %s:\n"),
 			   filename);
-		gtkgs_errmsg (gs, buf);
+		gtk_gs_emit_error_msg (gs, buf);
 		if (ggv_file_length (filename_err) > 0) {
 			FILE *err;
 			if ((err = fopen (filename_err, "r"))) {
 				/* print file to message window */
 				while (fgets (buf, 1024, err))
-					gtkgs_errmsg (gs, buf);
+					gtk_gs_emit_error_msg (gs, buf);
 				fclose (err);
 			}
 		}
@@ -1211,6 +1221,7 @@ static gchar *check_filecompressed (GtkGS *gs)
 	unlink (filename_err);
 	g_free (filename_err);
 	g_free (cmdline);
+	g_free (filename);
 	return filename_unc;
 }
 
@@ -1253,14 +1264,14 @@ static gchar *check_pdf (GtkGS *gs)
 			g_snprintf (buf, 1024,
 				    _("Error while scanning pdf file %s:\n"),
 				   filename);
-			gtkgs_errmsg (gs, buf);
+			gtk_gs_emit_error_msg (gs, buf);
 
 			if (ggv_file_length (filename_err) > 0) {
 				FILE *err;
 				if ((err = fopen (filename_err, "r"))) {
 					/* print file to message window */
 					while (fgets (buf, 1024, err))
-						gtkgs_errmsg (gs, buf);
+						gtk_gs_emit_error_msg (gs, buf);
 				}
 			}
 			unlink (filename_dsc);
@@ -1299,9 +1310,8 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
         gs->structured_doc = FALSE;
  	gs->send_filename_to_gs = TRUE;
 	gs->current_page = -1;
-
+        gs->loaded = FALSE;
 	gs->gs_filename = g_strdup (fname);
-	gs->current_page = -1;
 
 	if ((gs->reading_from_pipe = (strcmp (fname, "-") == 0))) {
                 gs->send_filename_to_gs = FALSE;
@@ -1316,7 +1326,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 			gchar buf[1024];
 			g_snprintf (buf, 1024,
 				    _("Cannot open file %s\n"), fname);
-			gtkgs_errmsg (gs, buf);
+			gtk_gs_emit_error_msg (gs, buf);
 		} else {
                         filename = check_filecompressed (gs);
 			if (filename)
@@ -1330,7 +1340,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 		}
 
                 /* we grab the vital statistics!!! */
-		gs->doc = psscan(gs->gs_psfile);
+		gs->doc = psscan(gs->gs_psfile, gs->respect_eof);
 
                 if (gs->doc == NULL) {
                         /* File does not seem to be a Postscript one */
@@ -1338,7 +1348,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 			gchar buf[1024];
 			g_snprintf (buf, 1024,
 				    _("Error while scanning file %s\n"), fname);
-			gtkgs_errmsg (gs, buf);
+			gtk_gs_emit_error_msg (gs, buf);
                         gtk_gs_cleanup (gs);
                         /*
                          * if (gdk_window_is_visible (gs->pstarget))
@@ -1347,24 +1357,49 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                         return FALSE;
                 }
 
-                /*
-                g_print("Page order %d\n", gs->doc->pageorder);
-                */
-                if ((!gs->doc->epsf && gs->doc->numpages>0) || 
-                    (gs->doc->epsf && gs->doc->numpages>1)) {
+                if ((!gs->doc->epsf && gs->doc->numpages > 0) || 
+                    (gs->doc->epsf && gs->doc->numpages > 1)) {
                         gs->structured_doc = TRUE; 
 			gs->send_filename_to_gs = FALSE;
 			gs->pages_marked = g_new0 (gint, gs->doc->numpages);
 		}
+                
+                /* We have to set up the orientation of the document */
+                
+                
+                /* orientation can only be portrait, and landscape or none.
+                 This is the document default. A document can have
+                 pages in landscape and some in portrait */
+                if (gs->override_orientation) {
+                        /* If the orientation should be override... 
+                           then gs->orientation has already the correct
+                           value (it was set when the widget was created */
+                        /* So do nothing */
 
-                /*
-                g_print(gs->structured_doc?"Struc, pages[%d]\n":"Non struc pages[%d]\n",
-                        gs->doc->numpages);
-                */
+                } else {
+                        /* Otherwise, set the proper orientation for the doc */
+                        switch  (gs->doc->orientation) {
+                        case LANDSCAPE:
+                                gs->real_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
+                                break;
+                        case PORTRAIT:
+                                gs->real_orientation = GTK_GS_ORIENTATION_PORTRAIT;
+                                break;
+                        case NONE:
+                                gs->real_orientation = GTK_GS_ORIENTATION_NONE;
+                                break;
+                        default:
+                                gs->real_orientation = GTK_GS_ORIENTATION_NONE;
+                                g_print("Orientation not handled yet %d\n",
+                                        gs->doc->orientation);
+                                break;
+                        }
+                }
                 stat(fname, &sbuf);
                 gs->mtime = sbuf.st_mtime;
   	}
 	gtk_widget_queue_resize (&(gs->widget));
+        gs->loaded = TRUE;
 
 	return TRUE;
 }
@@ -1416,19 +1451,17 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         struct stat sbuf;
 
 	g_assert (gs != NULL);
-
-        /*
-        g_print("gs goto page %d\n", page);
-        */
+        
         if (!gs->gs_filename) {
-                g_print ("filename is null???\n");
                 return FALSE;
         }
 
-        g_assert (GTK_WIDGET_REALIZED (gs));
+        if(!GTK_WIDGET_REALIZED(gs)) {
+                g_assert("GS not realized!");
+        }
         
         if (gs->gs_psfile) {
-        /* has it changed? */
+                /* has it changed? */
                 if (!stat(gs->gs_filename, &sbuf) &&
                     gs->mtime != sbuf.st_mtime) {
                         gchar *fname = g_strdup(gs->gs_filename);
@@ -1513,81 +1546,117 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
         gint new_lly;
         gint new_urx;
         gint new_ury;
-        gboolean from_doc = FALSE;
 
 	g_assert (gs != NULL);
 
-        if (new_pagemedia == -1) {
-                if (gs->doc && !gs->override_media) {
-                        /* We need to check that there are pages */
+        if(new_pagemedia == -1) {
+                new_pagemedia = gs->default_page_media;
+                if (!gs->override_media && gs->doc) {
+                        /* If we have a document:
+                           We use -- the page media (if specified)
+                                  or the doc. media (if specified)
+                                  or the page bbox (if specified)
+                                  or the bounding box
+                          */
                         if ((pageid >= 0) && (gs->doc->numpages > pageid) &&
                             (gs->doc->pages) && (gs->doc->pages[pageid].media)) {
                                 new_pagemedia = gs->doc->pages[pageid].media - gs->doc->media;
-                                from_doc = TRUE;
-                        } else if (gs->doc->default_page_media != NULL) {
-                                new_pagemedia = gs->doc->default_page_media - gs->doc->media;
-                                from_doc = TRUE;
-                        } else {
-                                new_pagemedia = gs->default_page_media;
                         }
-                } else {
-                        new_pagemedia = gs->default_page_media;
+                        else if (gs->doc->default_page_media != NULL) {
+                                new_pagemedia = gs->doc->default_page_media - gs->doc->media;
+                        }
+                        else if((pageid >= 0) &&
+                                (gs->doc->numpages > pageid) &&
+                                (gs->doc->pages) &&
+                                (gs->doc->pages[pageid].boundingbox[URX] >
+                                 gs->doc->pages[pageid].boundingbox[LLX]) &&
+                                (gs->doc->pages[pageid].boundingbox[URY] >
+                                 gs->doc->pages[pageid].boundingbox[LLY])) {
+                                new_pagemedia = -1;
+                        }
+                        else if((gs->doc->boundingbox[URX] > gs->doc->boundingbox[LLX]) &&
+                                (gs->doc->boundingbox[URY] > gs->doc->boundingbox[LLY])) {
+                                new_pagemedia = -1;
+                        }
                 }
-        }       
+        }
 
         /* Compute bounding box */
         if (!gs->doc) {
                 new_llx = new_lly = 0;
                 new_urx = papersizes[new_pagemedia].width;
                 new_ury = papersizes[new_pagemedia].height;
-        } else if (gs->doc->epsf &&
-                   (gs->doc->boundingbox[URX] > gs->doc->boundingbox[LLX]) &&
-            (gs->doc->boundingbox[URY] > gs->doc->boundingbox[LLY])) {
-                new_llx = gs->doc->boundingbox[LLX];
-                new_lly = gs->doc->boundingbox[LLY];
-                new_urx = gs->doc->boundingbox[URX];
-                new_ury = gs->doc->boundingbox[URY];
-        } else {
-
+        }
+        else if ((gs->doc->epsf || new_pagemedia == -1)) { /* epsf or bbox */
+                if((gs->doc->pages) &&
+                   (gs->doc->pages[pageid].boundingbox[URX] > gs->doc->pages[pageid].boundingbox[LLX]) &&
+                   (gs->doc->pages[pageid].boundingbox[URY] > gs->doc->pages[pageid].boundingbox[LLY])) {
+                        /* use page bbox */
+                        new_llx = gs->doc->pages[pageid].boundingbox[LLX];
+                        new_lly = gs->doc->pages[pageid].boundingbox[LLY];
+                        new_urx = gs->doc->pages[pageid].boundingbox[URX];
+                        new_ury = gs->doc->pages[pageid].boundingbox[URY];
+                }
+                else if((gs->doc->boundingbox[URX] > gs->doc->boundingbox[LLX]) &&
+                        (gs->doc->boundingbox[URY] > gs->doc->boundingbox[LLY])) {
+                        /* use doc bbox */
+                        new_llx = gs->doc->boundingbox[LLX];
+                        new_lly = gs->doc->boundingbox[LLY];
+                        new_urx = gs->doc->boundingbox[URX];
+                        new_ury = gs->doc->boundingbox[URY];
+                }
+                new_pagemedia = gs->default_page_media;
+        }
+        else {
                 new_llx = new_lly = 0;
-                if (gs->doc->media && (new_pagemedia < gs->doc->nummedia)) {
+                if (!gs->override_media && gs->doc->media &&
+                    (new_pagemedia < gs->doc->nummedia)) {
                         new_urx = gs->doc->media[new_pagemedia].width;
                         new_ury = gs->doc->media[new_pagemedia].height;
                 } else {
-                        new_urx = papersizes[new_pagemedia-gs->doc->nummedia].width;
-                        new_ury = papersizes[new_pagemedia-gs->doc->nummedia].height;
+                        new_urx = papersizes[new_pagemedia].width;
+                        new_ury = papersizes[new_pagemedia].height;
                 }
         }
 
         /* If bounding box changed, setup for new size. */
+                /* gtk_gs_disable_interpreter (gs); */
         if ((new_llx != gs->llx) || (new_lly != gs->lly) ||
             (new_urx != gs->urx) || (new_ury != gs->ury)) {
-                /* gtk_gs_disable_interpreter (gs); */
                 gs->llx = new_llx;
                 gs->lly = new_lly;
                 gs->urx = new_urx;
                 gs->ury = new_ury;
                 gs->changed = TRUE;
-
-		if (GTK_WIDGET_REALIZED (gs)) {
+        }
+   
+        if (gs->changed) {
+                if(GTK_WIDGET_REALIZED (gs)) {
                         set_up_page(gs);
-		gtk_widget_queue_resize (&(gs->widget));
-	}
+                        gtk_widget_queue_resize (&(gs->widget));
+                }
                 return TRUE;
-	}
+        }
+
         return FALSE;
 }
 
 gboolean
 gtk_gs_set_orientation (GtkGS *gs, gint orientation) {
+
+        gint iOldOrientation = gtk_gs_get_orientation(gs);
+        gs->fallback_orientation = orientation;
+                
 	g_assert (gs != NULL);
-        g_assert ((orientation == GTK_GS_ORIENTATION_DEFAULT) ||
+        g_assert (
                   (orientation == GTK_GS_ORIENTATION_PORTRAIT) ||
                   (orientation == GTK_GS_ORIENTATION_LANDSCAPE) ||
                   (orientation == GTK_GS_ORIENTATION_UPSIDEDOWN) ||
-                  (orientation == GTK_GS_ORIENTATION_SEASCAPE));
-        if (gs->orientation != orientation) { 
-                gs->orientation = orientation;
+                  (orientation == GTK_GS_ORIENTATION_SEASCAPE)
+                  );
+
+        /* We are setting the fallback orientation */
+        if (iOldOrientation != gtk_gs_get_orientation(gs)) {
                 gs->changed = TRUE;
                 if(GTK_WIDGET_REALIZED(gs))
                         set_up_page(gs);
@@ -1597,13 +1666,32 @@ gtk_gs_set_orientation (GtkGS *gs, gint orientation) {
         return FALSE;
 }
 
+void
+gtk_gs_set_override_orientation (GtkGS *gs, gboolean bNewOverride) {
+        
+        gint iOldOrientation = gtk_gs_get_orientation(gs);
+	g_assert (gs != NULL);
+
+        gs->override_orientation = bNewOverride;
+        
+        /* If the current orientation is different from the 
+           new orientation  then redisplay */
+        if (iOldOrientation != gtk_gs_get_orientation(gs)) {
+                gs->changed = TRUE;
+                if(GTK_WIDGET_REALIZED(gs))
+                        set_up_page(gs);
+        }
+	gtk_widget_queue_resize (&(gs->widget));
+}
+
 void
 gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
 	g_assert (gs != NULL);
 
         if (gs->zoom_factor != zoom) { 
                 gs->zoom_factor = zoom;
-                set_up_page(gs);
+                if (GTK_WIDGET_REALIZED(gs)) 
+                        set_up_page(gs);
                 gs->changed = TRUE;
         }
 	gtk_widget_queue_resize (&(gs->widget));
@@ -1628,33 +1716,33 @@ compute_size(GtkGS *gs)
 {
         guint new_width;
         guint new_height;
-        gboolean change;
+        gboolean change = FALSE;
+        gint orientation;
 
 	/* width and height can be changed, calculate window size according */
 	/* to xpdi and ydpi */
-	switch (gs->orientation) {
-                /* FIXME: what if the default orientation isn't normal??? */
-        case GTK_GS_ORIENTATION_DEFAULT:
+        orientation = gtk_gs_get_orientation(gs);
+
+        switch(orientation) {
         case GTK_GS_ORIENTATION_PORTRAIT:
-	case GTK_GS_ORIENTATION_UPSIDEDOWN:
+        case GTK_GS_ORIENTATION_UPSIDEDOWN:
                 new_width = (gs->urx - gs->llx) / 72.0 *
                         gs->xdpi + 0.5;
                 new_height = (gs->ury - gs->lly) / 72.0 *
                         gs->ydpi + 0.5;
                 break;
-	case GTK_GS_ORIENTATION_LANDSCAPE:
-	case GTK_GS_ORIENTATION_SEASCAPE:
+        case GTK_GS_ORIENTATION_LANDSCAPE:
+        case GTK_GS_ORIENTATION_SEASCAPE:
                 new_width = (gs->ury - gs->lly) / 72.0 *
                         gs->xdpi + 0.5;
                 new_height = (gs->urx - gs->llx) / 72.0 *
                         gs->ydpi + 0.5;
                 break;
-                
-	}
+        }
+
         change = (new_width != gs->width * gs->zoom_factor) || (new_height != gs->height * gs->zoom_factor);
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
-        /*  g_print("GS: width %d height %d \n", gs->width, gs->height); */
         if(GTK_WIDGET_REALIZED(gs)) {
                 if(!gdk_window_is_visible(gs->pstarget) && gs->width > 0 && gs->height > 0)
                         gdk_window_show(gs->pstarget);
@@ -1695,13 +1783,14 @@ gtk_gs_get_orientation(GtkGS *gs)
 	g_return_val_if_fail(gs != NULL, -1);
 	g_return_val_if_fail(GTK_IS_GS(gs), -1);
         
-        if (gs->orientation != GTK_GS_ORIENTATION_DEFAULT)
-                return gs->orientation;
-        /* FIXME: we want a way to get the orientation from the page */
-        return GTK_GS_ORIENTATION_PORTRAIT;
+        if (gs->override_orientation ||
+            gs->real_orientation == GTK_GS_ORIENTATION_NONE)
+                return gs->fallback_orientation;
+        else
+                return gs->real_orientation;
 }
 
-gchar *
+const gchar *
 gtk_gs_document_title(GtkGS *gs)
 {
 	g_return_val_if_fail(gs != NULL, NULL);
@@ -1725,7 +1814,7 @@ gtk_gs_document_numpages(GtkGS *widget)
 	return 0;
 }
 
-gchar *
+const gchar *
 gtk_gs_document_page_label(GtkGS *widget, int page)
 {
 	g_return_val_if_fail(widget != NULL, NULL);
@@ -1755,3 +1844,111 @@ gtk_gs_count_marked_pages(GtkGS *widget)
                         count++;
         return count;
 }
+
+void
+gtk_gs_set_default_orientation(gint iNewOrientation) {
+        g_assert (
+                  (iNewOrientation == GTK_GS_ORIENTATION_PORTRAIT) ||
+                  (iNewOrientation == GTK_GS_ORIENTATION_LANDSCAPE) ||
+                  (iNewOrientation == GTK_GS_ORIENTATION_UPSIDEDOWN) ||
+                  (iNewOrientation == GTK_GS_ORIENTATION_SEASCAPE)
+                  );
+        gtkgs_defaults.fallback_orientation = iNewOrientation;
+}
+
+void
+gtk_gs_set_default_page_media(gint iNewPageMedia)
+{
+        gtkgs_defaults.default_page_media = iNewPageMedia;
+}
+
+gint
+gtk_gs_get_default_page_media()
+{
+        return gtkgs_defaults.default_page_media;
+}
+
+void
+gtk_gs_set_default_watch_doc(gint iNewWatchDoc)
+{
+        gtkgs_defaults.watch_doc = iNewWatchDoc;
+}
+
+
+gint
+gtk_gs_get_default_watch_doc()
+{
+        return gtkgs_defaults.watch_doc;
+}
+
+
+
+gboolean
+gtk_gs_get_default_override_media()
+{
+        return gtkgs_defaults.override_media;
+}
+
+void
+gtk_gs_set_default_override_media(gboolean bOverMedia)
+{
+        gtkgs_defaults.override_media = bOverMedia;
+}
+
+
+
+gboolean
+gtk_gs_get_default_override_orientation()
+{
+        return gtkgs_defaults.override_orientation;
+}
+
+void
+gtk_gs_set_default_override_orientation(gboolean bOverOrien)
+{
+        gtkgs_defaults.override_orientation = bOverOrien;
+}
+
+gboolean
+gtk_gs_get_default_antialiased()
+{
+        return gtkgs_defaults.antialiased;
+}
+
+void
+gtk_gs_set_default_antialiased(gint iNewAntialiased)
+{
+        gtkgs_defaults.antialiased = iNewAntialiased;
+}
+
+
+gint
+gtk_gs_get_default_orientation()
+{
+        return gtkgs_defaults.fallback_orientation;
+}
+
+
+gfloat
+gtk_gs_get_default_zoom_factor()
+{
+        return gtkgs_defaults.zoom_factor;
+}
+
+void
+gtk_gs_set_default_zoom_factor(gfloat fZoom)
+{
+        gtkgs_defaults.zoom_factor = fZoom;
+}
+
+gboolean
+gtk_gs_get_default_respect_eof()
+{
+        return gtkgs_defaults.respect_eof;
+}
+
+void
+gtk_gs_set_default_respect_eof(gboolean resp)
+{
+        gtkgs_defaults.respect_eof = resp;
+}
